# coding:utf-8
#from https://newtechnologylifestyle.net/keras%E3%81%AEcnn%E3%82%92%
#             E4%BD%BF%E7%94%A8%E3%81%97%E3%81%A6%E3%82%AA%E3%83%AA%E3%
#             82%B8%E3%83%8A%E3%83%AB%E7%94%BB%E5%83%8F%E3%81%A7%E7%94%
#             BB%E5%83%8F%E8%AA%8D%E8%AD%98%E3%82%92%E8%A1%8C%E3%81%A3/ 
#input data from CIFAR-10
#  modified by M. Kogoshi & S. Kogoshi

import keras
from keras.utils import np_utils
from keras.models import Sequential, model_from_json
from keras.layers.convolutional import Conv2D, MaxPooling2D
from keras.layers.core import Dense, Dropout, Activation, Flatten
import numpy as np
from sklearn.model_selection import train_test_split
from PIL import Image
from keras.preprocessing.image import load_img, img_to_array
import glob

folder = ["bird","dog","car","ship"]
image_size = 32
 
X = []
Y = []
for index, name in enumerate(folder):
    dir = "./" + name
    files = glob.glob(dir + "/*.jpg")
    for i, file in enumerate(files):
        image = Image.open(file)
        image = image.convert("RGB")
        image = image.resize((image_size, image_size))
        data = np.asarray(image)
        X.append(data)
        Y.append(index)
 
X = np.array(X)
Y = np.array(Y)

X = X.astype('float32')
X = X / 255.0

# 正解ラベルの形式を変換
Y = np_utils.to_categorical(Y, 4)

train_images, valid_images, train_labels, valid_labels = train_test_split(X, Y, test_size=0.10)
X_train = train_images
y_train = train_labels
X_test = valid_images
y_test = valid_labels

# CNNを構築
model = Sequential()
 
model.add(Conv2D(32, (3, 3), padding='same',input_shape=X_train.shape[1:]))
model.add(Activation('relu'))
model.add(Conv2D(32, (3, 3)))
model.add(Activation('relu'))
model.add(MaxPooling2D(pool_size=(2, 2)))
model.add(Dropout(0.25))
 
model.add(Conv2D(64, (3, 3), padding='same'))
model.add(Activation('relu'))
model.add(Conv2D(64, (3, 3)))
model.add(Activation('relu'))
model.add(MaxPooling2D(pool_size=(2, 2)))
model.add(Dropout(0.25))
 
model.add(Flatten())
model.add(Dense(512))
model.add(Activation('relu'))
model.add(Dropout(0.5))
model.add(Dense(4))
model.add(Activation('softmax'))
 
# コンパイル
model.compile(loss='categorical_crossentropy',optimizer='SGD',metrics=['accuracy'])

#訓練
history = model.fit(X_train, y_train, epochs=200)

open("model.json","w").write(model.to_json())
model.save_weights("weight.hdf5")

#評価 & 評価結果出力
print(model.evaluate(X_test, y_test))

#****学習と予測を分離する場合はここで分け,##を消す************
#
##import keras
##from keras.utils import np_utils
##from keras.models import Sequential, model_from_json
##from keras.layers.convolutional import Conv2D, MaxPooling2D
##from keras.layers.core import Dense, Dropout, Activation, Flatten
##import numpy as np
##from sklearn.model_selection import train_test_split
##from PIL import Image
##from keras.preprocessing.image import load_img, img_to_array
##import glob

#予測
labels =['bird','dog','car','ship']

img = load_img('image.jpg')
img = img_to_array(img)
img = img.astype('float32')/255.0
img = np.array([img])

y_pred = model.predict(img)
number_pred = np.argmax(y_pred)
print("y_pred:", y_pred)
print("number_pred:", number_pred)
print('label_pred:', labels[int(number_pred)])